/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.DCDialog;
import net.sf.dc.console.components.Layout;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.db.QueryOptions;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.resources.DCTooltips;
import net.sf.dc.wf.requests.FillTableRequest;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * Creates a panel for creating the search condition. The search condition exists 
 * out of fields with a value. On this combination will be searched... 
 * Hidden fields are not shown in this panel. Also support searching parent items
 * based on a condition on a child item
 * 
 * @author Robert-Jan van der Waals
 * @since 1.5
 * @version 1.50
 */
public class SearchForm extends DCDialog {

    private final Vector vInput = new Vector(0);
    private final Vector vLabels = new Vector(0);
    
    private final DCModule[] modules;
    private final TablePanel parent;
    private int[] moduleIndices;
    
    private final SearchConditionPanel panelConditions;
    private final SearchOptionsPanel panelOptions;
    
    /**
     * Initializes this panel and set the correct fields for searching.
     * Hidden fields are not shown as well notsearchable fields.
     * In case of a child module, the fields of both the parent and the child
     * are shown.
     * @param module the module for which this panel is created
     */
    public SearchForm(DCModule module, TablePanel parent) {
        super(DataCrow.mainFrame);
        setTitle(Resources.getText(DCLabels.lblSearchForm));
        setHelpIndex("dc.Searching");
        
        this.parent = parent;
        
        if (module != null && module.getChildModule() != null) {
            modules = new DCModule[2];
            modules[0] = module;
            modules[1] = module.getChildModule();
        } else if (module != null) {
        	modules = new DCModule[1];
            modules[0] = module;
        } else {
            modules = new DCModule[0];
        }

        panelConditions = new SearchConditionPanel();
        panelOptions = new SearchOptionsPanel(module);
        
        buildForm(); 
        
        setBackground(Settings.getValueAsColor(Repository.settings.stColor));
        
        Dimension dim = Utilities.getCenteredWindowLocation(getSize());
        setLocation(dim.width, dim.height);
        setModal(true);
    }
    
    /**
     * Searches for the Software objects by using the conditions and options
     * as specified.
     */
    public void searchValues() {
        DCObject dco = panelConditions.getSearchValues();
        dco.addRequest(new FillTableRequest(dco.getModule().getIndex()));
        dco.load(panelOptions.getQueryOptions());
        parent.setStatus(Resources.getText(DCMessages.msgSearchHasBeenExecuted));
        
        if (parent.hasChild()) {
            parent.getChildModule().getSearchPanel().clearTable();
        }

       // try {
        DataCrow.mainFrame.setSelectedTab(0);
       // } catch (Exception ignore) {} // might not have initialized!
    }     
    
    private void buildForm() {
        getContentPane().setLayout(Layout.getGBL());
        getContentPane().add( panelConditions, Layout.getGBC( 0, 0, 2, 1, 2.0, 2.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
							  new Insets(5, 5, 5,  5), 0, 0));
        
        getContentPane().add( panelOptions, Layout.getGBC( 0, 1, 2, 1, 2.0, 2.0
        		             ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
							  new Insets(5, 5, 5,  5), 0, 0));
 
        JPanel panelActions = new JPanel();
        JButton buttonSearch = ComponentFactory.getButton(Resources.getText(DCLabels.lblFind));
        buttonSearch.setToolTipText(Resources.getText(DCTooltips.tpSearch));
        buttonSearch.addActionListener(new SearchButtonAction());
        buttonSearch.requestFocus();
 
        JButton buttonClose = ComponentFactory.getButton(Resources.getText(DCLabels.lblClose));
        buttonClose.addActionListener(new CloseButtonAction());
        
        panelActions.add(buttonSearch);
        panelActions.add(buttonClose);
        
        getContentPane().add( panelActions, Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                             ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                              new Insets( 5, 5, 5, 5), 0, 0));
        
        panelConditions.removeHiddenFields();
        
        pack();     
    }

    /**
     * The search criteria can be defined here  
     */
    private class SearchConditionPanel extends JPanel {
        
        public SearchConditionPanel() {
            buildDialog();         
        }
        
        /**
         * Makes sure that hidden fields are not displayed
         */
        public void removeHiddenFields() {
            // first, set all fields to be visible
            for (int i = 0; i < vInput.size(); i++) {
                JComponent comp = (JComponent) vInput.elementAt(i);
                comp.setVisible(true);
                JLabel label = (JLabel) vLabels.elementAt(i);
                label.setVisible(true);                
            }
            
            int diff = 0;
            for (int i = 0; i < modules.length; i++) {
                DCObject dco = modules[i].getDCObject();
                
                int[] hiddenFields = dco.getHiddenFields();
                int[] notSearchableFields = dco.getNotSearchableFields();
                
                for (int j = 0; j < hiddenFields.length; j++) {
                    JComponent comp = (JComponent) vInput.elementAt(hiddenFields[j] + diff);
                    comp.setVisible(false);
                    JLabel label = (JLabel) vLabels.elementAt(hiddenFields[j] + diff);
                    label.setVisible(false);    
                }
                
                for (int j = 0; j < notSearchableFields.length; j++) {
                    JComponent comp = (JComponent) vInput.elementAt(notSearchableFields[j] + diff);
                    comp.setVisible(false);
                    JLabel label = (JLabel) vLabels.elementAt(notSearchableFields[j] + diff);
                    label.setVisible(false);
                }
                diff = diff + dco.getFields().length;
            }
        }
                
        /**
         * Returns the values on which will be searched (the search condition)
         */
        public DCObject getSearchValues() {
            DCObject[] objects = new DCObject[modules.length]; 
            
            int diff = 0;
            for (int j = 0; j < modules.length; j++) {
                DCObject dco = modules[j].getDCObject();
                
                Object[] values = dco.getValues();
                int[] visibleFields = dco.getVisibleFields();
                int[] fieldTypes = dco.getFieldTypes();
        
                for (int i = 0; i < visibleFields.length; i++) {
                    int fieldType = fieldTypes[visibleFields[i]];
                    JComponent component = (JComponent) vInput.elementAt(visibleFields[i]  + (diff));
                    
                    if (fieldType == ComponentFactory._LONGTEXTFIELD) {
                        values[visibleFields[i]] = ComponentFactory.getComponentValue(
                                                        ComponentFactory._SHORTTEXTFIELD,
                                                        component);
                    } else {
                        values[visibleFields[i]] = ComponentFactory.getComponentValue(
                                                        fieldType,
                                                        component);    
                    }
                }
                
                for (int i = 0; i < values.length; i++) {
                    if (dco.isSearchable(i)) {
                        String s = values[i] == null ? "" : values[i].toString();
                        if (s != null && !s.equals("")) {
                            dco.setValue(i, values[i]);
                        }
                    }
                }
                
                objects[j] = dco;
                diff = diff + dco.getFields().length;
            }
            
            if (objects.length > 1) {
                objects[0].addChild(objects[1]);
            }
            return objects[0];
        }
        
        /**
         * Creates the panel
         */
        private void buildDialog() {
            setLayout(Layout.getGBL());
            
            int length = 0;
            for (int i = 0; i < modules.length; i++) {
                length += modules[i].getFieldCount();
            }
            
            moduleIndices = new int[length];
            
            int y = 0;
            for (int i = 0; i < modules.length; i++) {
                DCObject dco = modules[i].getDCObject();
                int[] fields = modules[i].getDCObject().getFieldIndices();
                
                for (int j = 0; j < fields.length; j++) {
                    int field = fields[j];
                    JLabel label = ComponentFactory.getLabel(dco.getLabel(field));
                    
                    JComponent input = null;
                    if (dco.getFieldType(field) == ComponentFactory._LONGTEXTFIELD) {
                        input = ComponentFactory.getComponent(
                                ComponentFactory._SHORTTEXTFIELD, 
                                dco.getLabel(field),
                                dco.getMaxFieldLength(field)); 
                    } else {
                        input = ComponentFactory.getComponent(
                                dco.getFieldType(field), 
                                dco.getLabel(field),
                                dco.getMaxFieldLength(field)); 
                    }
                    
                    input.setPreferredSize(new Dimension(200,20));
                    label.setPreferredSize(new Dimension(150,20));

                    int bottom = 0;
                    if (i == fields.length -1) {
                        bottom = 0;   
                    }

                    add( label, Layout.getGBC( 0, y, 1, 1, 1.0, 1.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                          new Insets(0,  5, bottom, 5), 0, 0));
                    add( input, Layout.getGBC( 1, y, 1, 1, 2.0, 2.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                          new Insets(0, 5, bottom,  5), 0, 0));

                    vLabels.addElement(label);
                    vInput.addElement(input);
                    
                    // not the real module index, but the index for the modules array
                    moduleIndices[y] = i;
                    y++;
                }
            }
        }         
    }
    
    /**
     * Search Option panel. Here options, such as 
     * comply to all conditions, are maintained
     */
    public class SearchOptionsPanel extends JPanel {
        private JCheckBox checkComplyAll = 
            ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblAndOrSelect));
        private JCheckBox checkPrecise =
            ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblPreciseSelect));
        private JLabel labelOrderBy = ComponentFactory.getLabel(
                                      Resources.getText(DCLabels.lblOrderBy));
        private JComboBox comboOrderBy = ComponentFactory.getComboBox();

        private DCObject dco;
        
        public SearchOptionsPanel(DCModule module) {
            dco = module.getDCObject();
            setLayout(Layout.getGBL());

            String[] colNames    = dco.getLabels();
            comboOrderBy.addItem("");
            for (int i = 0; i < colNames.length; i++) {
                comboOrderBy.addItem(colNames[i]);
            }
            
            comboOrderBy.setSelectedItem(colNames[1]);
            checkComplyAll.setSelected(true);
            
            add( checkComplyAll, Layout.getGBC( 0, 0, 2, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 3, 5), 0, 0));
            add( checkPrecise,   Layout.getGBC( 0, 1, 2, 1, 5.0, 5.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 3, 5), 0, 0));
            add( labelOrderBy,   Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 0, 5), 0, 0));
            add( comboOrderBy,   Layout.getGBC( 1, 2, 1, 1, 5.0, 5.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                 new Insets( 0, 5, 5, 5), 0, 0));
        }

        /**
         * Returns the query options used for searching values
         */
        public QueryOptions getQueryOptions() {
            boolean bPreciseSelect = checkPrecise.isSelected();
            boolean bComplyToAllCondition = checkComplyAll.isSelected();
            String sOrderBy = "";
            if (comboOrderBy.getSelectedIndex() > 0) {
                sOrderBy = dco.getModule().getFieldDefinition(comboOrderBy.getSelectedIndex() - 1).getDatabaseFieldName();
            }
            return new QueryOptions(sOrderBy, bComplyToAllCondition, bPreciseSelect);
        }
    }  

    public class CloseButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            setVisible(false);
        }
    } 
    
    public class SearchButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            searchValues();
        }
    }    
}
