/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.DCDialog;
import net.sf.dc.console.components.Layout;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * This dialog allows the user to apply a value on a field 
 * to all items on screen
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.90
 */
public class UpdateAllDialog extends DCDialog {
    
    private JButton buttonApply;
    private JButton buttonCancel;
    
    private JLabel[] cLabels;
    private JComponent[] cInputFields;
    
    private DCObject o;
    private boolean success = false;
   
    /**
     * Creates and shows the update all dialog
     * @param o Data Crow Object used as a reference
     * @param parent the parent frame
     */
    public UpdateAllDialog(DCObject o, JFrame parent) {
        super(parent);

        setHelpIndex("dc.Tools.UpdateAll");
        
        setTitle(Resources.getText(DCLabels.lblUpdateAll));
        this.o = o;
        buildDialog(o);
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Settings.setValue(Repository.settings.stUpdateAllDialogSize,
                                  getSize());
                success = false;
                done();
            }
        });

        setSize(Settings.getValueAsDimension(
                Repository.settings.stUpdateAllDialogSize));

        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);

        Color color = Settings.getValueAsColor(Repository.settings.stColor);
        setBackground(color);
        
        setModal(true);        
        setVisible(true);
    }
    
    /**
     * Returns if the update all action was successfull
     */
    public boolean isSuccess() {
        return success;   
    }

    /**
     * Returns the values. These values should be used to update 
     * all the items 
     */
    public Object[] getData() {
        Object[] values = new Object[cInputFields.length];
        values[0] = "";
        for (int i = 1; i < o.getFieldIndices().length; i++) {
            int fieldType = o.getFieldType(i) == ComponentFactory._LONGTEXTFIELD ? 
                                                 ComponentFactory._SHORTTEXTFIELD :
                                                 o.getFieldType(i);
            
            Object value = ComponentFactory.getComponentValue(
                           fieldType, 
                           cInputFields[i]);
            values[i] = value;
        }
        return values;
    }  
    
    /**
     * Closes this dialog
     */
    private void done() {
        setModal(false);
        dispose();
    }
    
    /**
     * Hidden fields are not displayed
     * @param indices
     */
    private void hideFields(int[] indices) {
        cLabels[0].setVisible(false);
        cInputFields[0].setVisible(false);
        for (int i = 0; i < indices.length; i++) {
            cLabels[indices[i]].setVisible(false);
            cInputFields[indices[i]].setVisible(false);
        }
    }     
    
    private void buildDialog(DCObject o) {
        Color color = Settings.getValueAsColor(Repository.settings.stColor);
        cLabels = new JLabel[o.getFieldTypes().length];
        cInputFields = new JComponent[o.getFieldTypes().length];
        
        //**********************************************************
        //Input panel
        //**********************************************************
        JPanel panelInput = new JPanel(false);
        panelInput.setLayout(Layout.getGBL());
        panelInput.setBackground(color);

        for (int i = 0; i < o.getFieldTypes().length; i ++) {
            cLabels[i]      = ComponentFactory.getLabel(o.getLabel(i));
            
            int fieldType = o.getFieldType(i);
            if (fieldType == ComponentFactory._LONGTEXTFIELD) {
                fieldType = ComponentFactory._SHORTTEXTFIELD;   
            }
            
            cInputFields[i] = ComponentFactory.getComponent(fieldType, 
                                                                o.getLabel(i),
                                                                o.getMaxFieldLength(i));
            
            panelInput.add(cLabels[i]        , Layout.getGBC(0, i, 1, 1, 1.0, 1.0
                                             , GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                               new Insets(0, 5, 0, 5), 0, 0));            
            panelInput.add(cInputFields[i]   , Layout.getGBC(1, i, 1, 1, 10.0, 10.0
                                             , GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                               new Insets(0, 5, 0, 5), 0, 0));
        }
        
        //**********************************************************
        //Action panel
        //**********************************************************
        JPanel panelActions = new JPanel(false);
        panelActions.setLayout(Layout.getGBL());
        panelActions.setBackground(color);

        buttonApply = ComponentFactory.getButton(Resources.getText(DCLabels.lblApply));
        buttonCancel = ComponentFactory.getButton(Resources.getText(DCLabels.lblCancel));
        
        buttonApply.addActionListener(new ApplyAction());
        buttonCancel.addActionListener(new CancelAction());
        
        buttonApply.setMnemonic(KeyEvent.VK_A);
        buttonCancel.setMnemonic(KeyEvent.VK_C);
        
        panelActions.add(buttonApply    , Layout.getGBC(0, 0, 1, 1, 0.0, 0.0
                                        , GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets(5, 5, 5, 5), 0, 0));            
        panelActions.add(buttonCancel   , Layout.getGBC(1, 0, 1, 1, 0.0, 0.0
                                        , GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets(5, 5, 5, 5), 0, 0));
                                          
        //**********************************************************
        //Main panel
        //**********************************************************
        this.getContentPane().setLayout(Layout.getGBL());
        this.getContentPane().add(panelInput  ,Layout.getGBC(0, 0, 1, 1, 10.0, 10.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                               new Insets(5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panelActions,Layout.getGBC(0, 1, 1, 1, 1.0, 1.0
                                              ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                                               new Insets(5, 5, 5, 5), 0, 0));
        
        this.getContentPane().setBackground(color);
        pack();
        hideFields(o.getHiddenFields());
        hideFields(o.getNotSearchableFields());
    }
    
    private class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            success = false;
            done();
        }
    }
    
    private class ApplyAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            success = true;
            done();
        }
    }   
}
