/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core;

import java.io.File;
import java.util.ResourceBundle;

import net.sf.dc.console.MainFrame;
import net.sf.dc.console.SplashScreen;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.core.modules.AudioCDModule;
import net.sf.dc.core.modules.AudioTrackModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.modules.MovieModule;
import net.sf.dc.core.modules.MusicFileModule;
import net.sf.dc.core.modules.SoftwareCategoryModule;
import net.sf.dc.core.modules.SoftwareModule;
import net.sf.dc.core.objects.IconCache;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.resources.DCRIFFAudioCodecs;
import net.sf.dc.resources.DCTooltips;
import net.sf.dc.settings.SettingInitializator;
import net.sf.dc.wf.WorkFlow;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.ModularResourceBundle;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * DataCrow is the starting point of the application. 
 * It is responsible for initializing the UI, starting the 
 * logging, initialization of the settings, 
 * initializing the database connection and starting the UI.
 * 
 * There are no arguments needed for calling the main method, 
 * all will be ignored. Input for the program is located in the 
 * settings file (Datacrow.properties)
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class DataCrow {

    public static MainFrame mainFrame;
    private static SplashScreen splashScreen;

    public static void main(String[] args) {
        // check if the data directory exists. If not, create it
        File file = new File("./data");
        if (!file.exists()) {
            file.mkdir();
        }
        
        // check if the cache directory exists. If not, create it
        String sCacheDir = Utilities.getCacheDirectory();
        File f = new File(sCacheDir);
        if (!f.exists()) {
            f.mkdir();
        }
        
        DataCrow application = new DataCrow();
        
        // Initialize the Resources
        Resources.addTextResourceBundle((ModularResourceBundle) ResourceBundle.getBundle(DCTooltips.class.getName()));
        Resources.addTextResourceBundle((ModularResourceBundle) ResourceBundle.getBundle(DCLabels.class.getName()));
        Resources.addTextResourceBundle((ModularResourceBundle) ResourceBundle.getBundle(DCMessages.class.getName()));
        Resources.addTextResourceBundle((ModularResourceBundle) ResourceBundle.getBundle(DCRIFFAudioCodecs.class.getName()));
        
        showSplashScreen();
        
        splashScreen.setStatusMsg("Initializing settings");
        // Initialize all settings
        new SettingInitializator();        
        
        // Initialize the Component factory
        splashScreen.setStatusMsg("Loading components");
        new ComponentFactory();
        
        // Start the logging thread
        new Messages();
        Messages.add(Resources.getText(DCMessages.msgNewSession0) + '\n' +
                            Resources.getText(DCMessages.msgNewSession1) + '\n' +
                            Resources.getText(DCMessages.msgNewSession2) + '\n' +
                            Resources.getText(DCMessages.msgNewSession3) + '\n',
                            Messages._TABLESS);
        
		Messages.add(
            Resources.getText(DCMessages.msgApplicationStarts),
            Messages._STANDARD);
        
        

        // Initialize all modules
        application.registerModules();

        Messages.add(Resources.getText(DCMessages.msgSettingsLoaded),
                            Messages._STANDARD);
        
        // Establish a connection to the database / server
        splashScreen.setStatusMsg("Initializing database");
        WorkFlow.initDatabase();

        try {
            // Update the icon cache
        	IconCache.refresh();
            
            // Start the UI
            splashScreen.setStatusMsg("Loading UI");
            mainFrame = new MainFrame();
            mainFrame.setVisible(true);
            
            Thread splashCloser = new Thread(new SplashScreenCloser());
            splashCloser.start();

            // perform a blank search on startup
            application.blankSearch(
                    Settings.getValueAsInt(Repository.settings.stModule));
            
        } catch (Exception exp) {
            Messages.add(exp);
            exp.printStackTrace();
            System.exit(0);
        }
    }
    
    private void blankSearch(int module) {
        if (Settings.getValueAsBoolean(Repository.settings.stAutoLoadData)) {
            DCModules.getModule(module).getSearchForm().searchValues();
        }
    }       
    
    private void registerModules() {
        splashScreen.setStatusMsg("Loading software module");
        DCModules.register(new SoftwareCategoryModule());
        DCModules.register(new SoftwareModule());
        splashScreen.setStatusMsg("Loading movie module");
        DCModules.register(new MovieModule());
        splashScreen.setStatusMsg("Loading music module");
        DCModules.register(new MusicFileModule());
        
        splashScreen.setStatusMsg("Loading audio cd module");
        AudioTrackModule audioTrackModule = new AudioTrackModule();
        AudioCDModule audioCDModule = new AudioCDModule();
        
        audioCDModule.setChildModule(audioTrackModule);
        DCModules.register(audioCDModule);
        DCModules.register(audioTrackModule);
    }
    
    private static void showSplashScreen(){
    	splashScreen = new SplashScreen();
    	splashScreen.splash();
    }
    
    public static void closeSplashScreen() {
        splashScreen.dispose();
        splashScreen = null;
    }
    
    public static void moveSplashScreenToBack() {
        if (splashScreen != null) splashScreen.toBack();
    }

    public static void moveSplashScreenToFront() {
        if (splashScreen != null) splashScreen.toFront();
    }

    private static final class SplashScreenCloser implements Runnable {
    	public void run(){
    		splashScreen.dispose();
            splashScreen = null;
    	}
    }
}
