/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import javax.swing.ImageIcon;
import javax.swing.JMenu;

import net.sf.dc.console.DCMenuBar;
import net.sf.dc.console.DCPanel;
import net.sf.dc.console.SearchForm;
import net.sf.dc.console.TableInsertPanel;
import net.sf.dc.console.TableSearchPanel;
import net.sf.dc.console.da.ImportCDDialog;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.processes.cdimport.ImportCDProcess;

/**
 * Contains all generic information for this module. 
 * Only static data can be found in here, such as the column and
 * table information. The UI representation can also be retrieved
 * thru this class (insert and search panels)
 * 
 * Hidden field information and such, which can be changed during
 * a session is contained within the Data Crow Objects themselves.
 * 
 * Main reason for this setup is to keep the Data Crow Objects as
 * small as possible, they can be stored in memory by thousands
 * at a time..
 * 
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 1.00
 */
public abstract class DCModule {
    
    protected TableInsertPanel insertPanel;
    protected TableSearchPanel searchPanel;
    protected SearchForm searchForm;
    
    protected DCModule childModule;
    protected DCFieldDefinition[] fields;
    
    private final int index;
    private final String name;
    
    private final boolean allowImdbUpdate;
    private final boolean allowAmazonUpdate;
    
    protected String tableName = "";
    protected String tableShortName = "";
    protected String tableJoin = "";
    
    /**
     * Initializes this module
     * 
     * @param index static final module index
     * @param name name of this module
     * @param allowImdbUpdate internet updates, thru imdb, 
     *        allowed y/n
     * @param allowAmazonUpdate internet updates, thru amazon, 
     *        allowed y/n
     * @param standalone is this a standalone or a composite module 
     */
    public DCModule(        int index, 
                            String name,
                            String tableName,
                            String tableShortName,
                            String tableJoin,
                            boolean allowImdbUpdate,
                            boolean allowAmazonUpdate) {
        this.index = index;
        this.tableName = tableName;
        this.tableShortName  = tableShortName;
        this.tableJoin = tableJoin;
        this.name = name;
        this.allowAmazonUpdate = allowAmazonUpdate;
        this.allowImdbUpdate = allowImdbUpdate;
        initializeFields();
    }
    
    /**
     * Returns all insert and search panels belonging to this module
     * @return array of panels
     */
    public DCPanel[] getPanels() {
        if (getSearchPanel() != null && getInsertPanel() != null) {
            return new DCPanel[] {getSearchPanel(), getInsertPanel()};
        } else if (getSearchPanel() != null) {
             return new DCPanel[] {getSearchPanel()};
        } else if (getInsertPanel() != null) {
            return new DCPanel[] {getInsertPanel()};
        } else {
        	return new DCPanel[0];
        }
    }
    
    /**
     * Returns a specific field definition. A field definition
     * describes if the field is required, what datatype it represents,
     * etc.
     * @param index field index
     */
    public DCFieldDefinition getFieldDefinition(int index) {
        return fields[index];
    }

    /**
     * Sets a child module for this module
     * @param childModule child module
     */
    public void setChildModule(DCModule childModule) {
    	this.childModule = childModule;
    }
    
    /**
     * Returns the child module (or null)
     */
    public DCModule getChildModule() {
    	return childModule;
    }
    
    public SearchForm getSearchForm() {
    	return searchForm;
    }
    
    /**
     * Returns all field definitions. A field definition
     * describes if the field is required, what datatype it represents,
     * etc.
     */
    public DCFieldDefinition[] getFieldDefinitions() {
        return fields;
    }

    // to be implemented by children
    protected abstract void initializeFields();
    public abstract ImageIcon getIcon();
    public abstract String getHiddenFieldsSettingsKey();
    public abstract String getRequiredFieldsSettingsKey();
    public abstract String getObjectName();
    public abstract DCMenuBar getMenuBar();
    public abstract DCObject[] getDefaultData() throws Exception;
    public abstract void startImport();

    /**
     * Returns an empty Data Crow object, managed by this module 
     */
    public abstract DCObject getDCObject();
    
    /**
     * Returns module specific menu items  
     */
    public abstract JMenu getToolsMenu();
    
    /**
     * Creates the panels for this module
     */
    protected abstract void initializeUI();
    
    /**
     * Returns the module specific CD import process
     * 
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     */
    public abstract ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                                       String sourceDir,
													   boolean confirmEachDir,
													   String nameOfCD);

    /**
     * Returns the amount of fields 
     */
    public int getFieldCount() {
        return fields.length;    
    }
    
    /**
     * Initializes the insert panel (lazy)
     */
    public TableInsertPanel getInsertPanel() {
        if (insertPanel == null) {
            initializeUI();
        }
        return insertPanel;
    }
    
    /**
     * Initializes the search panel (lazy)
     */
    public TableSearchPanel getSearchPanel() {
        if (searchPanel == null) {
            initializeUI();
        }
        return searchPanel;
    }
    
    /**
     * Indicates if this module has an Amazon update implementation
     */
    public boolean isAmazonUpdateAllowed() {
        return allowAmazonUpdate;
    }

    /**
     * Indicates if this module has an IMDB update implementation
     */
    public boolean isImdbUpdateAllowed() {
        return allowImdbUpdate;
    }
    
    /**
     * Returns the display name of this module
     */
    public String getName() {
        return name;
    }
    
    public String getTableName() {
    	return tableName;
    }

    public String getTableShortName() {
        return tableShortName;
    }
    
    public String getTableJoin() {
    	return tableJoin;
    }
    
    /**
     * Returns the index for this module as defined in DCModules
     * @see DCModules 
     */
    public int getIndex() {
        return index;
    }
    
    /**
     * Creates a string representation for this module
     */
    public String toString() {
        return name;
    }
}
