/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.db;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;

import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.wf.WorkFlow;
import net.sf.dc.wf.requests.RequestCollection;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * Holder of the Database connection and information. Starts the
 * QueryQueue and handles update, insert and select statements.
 * 
 * On the first start, the Database will be created (if the database
 * is empty; the tables do not exist). If this fails, a message is
 * shown to the user.
 * 
 * For new itemss, the unique ID is created by this class. This has
 * not been perfected at this moment.
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class Database {

    private static Connection connection;
    private QueryQueue queue;

    /**
     * Initializes the Database, creates a new database if necessary
     */
    public Database() {
        init();
    }

    /**
     * Closes the database connection
     */
    public static void close() {
        try {
            connection.close();
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }
    
    public static boolean isClosed() {
        try {
        	return connection.isClosed();
        } catch (Exception exp) {
        	return true;
        }
    }
    
    /**
     * Starts the query queue and initializes the database connection
     *
     * @see DatabaseConnection
     * @see QueryQueue
     */
    public void init() {
        new DatabaseConnection();
        
        if (connection == null) {
            initializeDatabase();
        }
        
        queue = new QueryQueue(connection);
        Thread queryQueue = new Thread(queue, "queryQueue");

        queryQueue.setPriority(Thread.MAX_PRIORITY);
        queryQueue.setDaemon(true);
        queryQueue.start();
    }

    /**
     * Runs a query 
     * @param sQuery query statement (string)
     * @param requestors requests to the UI
     */
    public void runQuery(String sQuery, RequestCollection requestors) {
        Query query = new Query(null, null, null, Query._UNDEFINED, requestors, null);
        query.setQueryString(sQuery);
        queue.addQuery(query);
    }

    /**
     * Puts the query in the queue. The queue will handle the query when time
     * comes and returns the results (if there are any) to the requestor by using
     * the RequestorObject class
     * @see RequestorObject 
     * @see QueryQueue
     * @param query query object
     */
    private void putQueryInQueue(Query query) {
        queue.addQuery(query);
    }

    /**
     * Runs a query direct on the database instead of putting it in query 
     * queue. It returns an unclosed result set. 
     * @param sQuery query to be executed
     */
    public ResultSet runQueryDirectUnclosed(String sQuery, boolean silent) throws Exception {
        if (!silent) {
        	Messages.add(sQuery, Messages._QUERY);
        }
        Statement stmt = connection.createStatement();    
        return stmt.executeQuery(sQuery);
    }

    /**
     * Does not put the query in a queue but executes it immediately.
     * Errors are either thrown or returned to the caller
     * @param sQuery  query statement
     * @param catchErrors indicates whether errors should be caugth or passed on
     *                    to the caller  
     * @return list of data crow objects (can be null or empty)
     */    
    public Collection runQueryDirect(String sQuery, boolean catchErrors) throws Exception {
    	Collection data = null;
        try {
            Statement stmt = connection.createStatement();    
            ResultSet result = stmt.executeQuery(sQuery);
            data = WorkFlow.convertToDCObjects(result);
            stmt.close();
            result.close();
        } catch (SQLException sqlExp) {
            if (!sqlExp.getMessage().equals("No ResultSet was produced")) {
                if (catchErrors) {
                	Messages.add(sqlExp);
                } else {
                	throw sqlExp;
                }
            }
        }
    	Messages.add(sQuery, Messages._QUERY);
        return data;
    }
    
    /**
     * Does not put the query in a queue but executes it immediately.
     * Errors are returned to the caller (not thrown)
     * @param sQuery  query statement
     * @return list of data crow objects (can be null or empty)
     */
    public Collection runQueryDirect(String sQuery) {
        Collection objects = null;
        try {
            objects = runQueryDirect(sQuery, true);
            // errors are caught in the runQueryDirect method !!
        } catch (Exception ignore) {}
        return objects;
    }

    /**
     * Runs a query directly, bypassing the query queue
     * @param o data crow object
     */
    public Collection runQueryDirect(DCObject dco) {
        QueryWhat queryWhat = new QueryWhat(dco, dco.getDatabaseFieldNames());
        Query query = new Query(queryWhat, dco.toQueryCondition(true), null, Query._SELECT, null, null);
        Messages.add(query.getQueryString(), Messages._QUERY);
        return runQueryDirect(query.getQueryString());
    }

    /**
     * Does not put the query in a queue but executes it immediately.
     * Errors are returned to the caller (not thrown)
     * @param query Query object
     */
    public Collection runQueryDirect(Query query) {
        return runQueryDirect(query.getQueryString());
    }

    /**
     * Searches for values. The query is put in the query queue. The results are
     * past to the caller by using the RequestorCollection
     *
     * @param o the object which is used as the condition
     * @param options query options
     */
    public void searchValues(DCObject dco, QueryOptions options) {
        QueryWhat queryWhat = new QueryWhat(dco, dco.getDatabaseFieldNames());
        Query query = new Query(queryWhat, dco.toQueryCondition(true), options, Query._SELECT, dco.getRequests(), null);
        query.setSilence(dco.getSilence());
        putQueryInQueue(query);
    }

    /**
     * Updates values. The query is put in the query queue. Actions are
     * past to the caller by using the RequestorCollection
     * @param o  the object to be updated
     */
    public void updateValues(DCObject dco) {
        String sID = dco.getValue(DCObject._ID).toString();

        QueryWhat queryWhat = new QueryWhat(dco, dco.getDatabaseFieldNames());
        Query query = new Query(queryWhat, dco.toQueryCondition(false), null, Query._UPDATE, dco.getRequests(), sID);
        
        if (dco.isBatch()) {
        	query.setBatch(dco.isEndOfBatch());
        }
        
        query.setSilence(dco.getSilence());
        putQueryInQueue(query);
        
        Collection children = dco.getChildren();
        for (Iterator iter = children.iterator(); iter.hasNext(); ) {
            DCObject child = (DCObject) iter.next();
            queryWhat = new QueryWhat(child, child.getDatabaseFieldNames());
            query = new Query(queryWhat, child.toQueryCondition(false), 
                               null, Query._UPDATE, 
                               child.getRequests(), child.getID());
            query.setSilence(dco.getSilence());
            putQueryInQueue(query);
            child = null;
        }
        dco = null;       
    }

    /**
     * Inserts new values. The query is put in the query queue. Actions are
     * past to the caller by using the RequestorCollection
     * @param o the object to be inserted
     * @param requestors requestors which depend on the outcome of the query
     */
    public void insertValues(DCObject dco) {
        String sID = dco.getValue(DCObject._ID).toString();
        
        if (sID == null || sID.equals("")) {
            sID = Utilities.getUniqueID().substring(0,12);
        	dco.setValue(DCObject._ID, sID);
        }

        QueryWhat queryWhat = new QueryWhat(dco, dco.getChangedFields());
        Query query = new Query(queryWhat, dco.toQueryCondition(false), null, Query._INSERT, dco.getRequests(), sID);
        if (dco.isBatch()) {
        	query.setBatch(dco.isEndOfBatch());
        }
        
        query.setSilence(dco.getSilence());
        putQueryInQueue(query);
        
        Collection children = dco.getChildren();
        for (Iterator iter = children.iterator(); iter.hasNext(); ) {
            DCObject child = (DCObject) iter.next();
            queryWhat = new QueryWhat(child, child.getChangedFields());
            query = new Query(queryWhat, child.toQueryCondition(false), 
                               null, Query._INSERT, 
                               child.getRequests(), sID);
            query.setSilence(dco.getSilence());
            putQueryInQueue(query);
            child = null;
        }
        dco = null;
    }

    /**
     * Deletes values from the database. The query is put in the query queue. 
     * Actions are past to the caller by using the RequestorCollection
     * @param o the object to be deleted
     * @param requestors requestors which depend on the outcome of the query
     */
    public void deleteValues(DCObject dco) {
        String sID = dco.getValue(DCObject._ID).toString();

        QueryWhat queryWhat = new QueryWhat(dco, dco.getDatabaseFieldNames());
        Query query = new Query(queryWhat, dco.toQueryCondition(false), null, Query._DELETE, dco.getRequests(), sID);
        
        if (dco.isBatch()) {
            query.setBatch(dco.isEndOfBatch());
        }
        
        query.setSilence(dco.getSilence());
        putQueryInQueue(query);
        dco = null;
    }
    
    /**
     * Checks whether the values are unique (do not exist as yet in the database).
     * Updates and inserts are checked differently
     * @param o   object to be checked
     * @param bUpdateQuery  specifies whether this is an update or an insert
     */
    public boolean uniqueValues(DCObject o, boolean bUpdateQuery) {
        String sID = o.getValue(DCObject._ID).toString();

        int[] requiredFields = o.getRequiredFields();

        DCObject dco = o.getModule().getDCObject();
        for (int i = 0; i < requiredFields.length; i++) {
            dco.setValue(requiredFields[i], o.getValue(requiredFields[i]));
        }
        
        dco.setSilence(true);

        QueryCondition condition = new QueryCondition(dco, true);
        QueryOptions options = new QueryOptions(null, true, true);
        QueryWhat queryWhat = new QueryWhat(dco, dco.getDatabaseFieldNames());
        Query query = new Query(queryWhat, condition, options, Query._SELECT, null, "");
        query.setSilence(true);
        
        ResultSet result = null;
        try {
            result = runQueryDirectUnclosed(query.getQueryString(), true);
        } catch (Exception exp) {
            Messages.add(exp);
        }

        if (result == null) {
            return true;
        }

        int rows = 0;
        try {
            while (result.next()) {
                String itemID = result.getString(dco.getDatabaseFieldName(
                                                   DCObject._ID));
                if (!sID.equals(itemID)) {
                    rows++;
                }
            }
            result.close();
        } catch (SQLException sqlExp) {
            Messages.add(sqlExp);
        }
        
        dco = null;

        if (bUpdateQuery && rows >= 1) {
            return false; // Two rows or more with the same ID
        } else if (!bUpdateQuery && rows > 0) {
            return false; // One row exists already with the same ID
        }
        
        return true;
    }

    /**
     * Creates a connection to the database. If no database exsists, a new
     * one is created. Demo data is loaded when this is preferred. The login dialog
     * is used to get user input
     */
    private synchronized void initializeDatabase() {
        while (connection == null) {
            connection = DatabaseConnection.connect();
        }

        Statement stmt = null;
        try {
            stmt = connection.createStatement();
        } catch (Exception sqlExp) {
            return;
        }
        
        for (Enumeration enumModules = DCModules.getModules(); 
             enumModules.hasMoreElements(); ) {

            DCModule module = (DCModule) enumModules.nextElement();
            DCObject dco = module.getDCObject(); 
            
            try {
                String testQuery = "select * from " + dco.getTableName();
                stmt.setMaxRows(1);
                ResultSet result = stmt.executeQuery(testQuery);
                initializeColumns(result.getMetaData(), dco);
                Messages.add(Resources.getText(DCMessages.msgTableFound, dco.getTableName()), 
                                    Messages._STANDARD);
                result.close();
            } catch (SQLException exp) {
                Messages.add(Resources.getText(DCMessages.msgTableNotFound, dco.getTableName()), 
                                    Messages._STANDARD);
                initializeNewDatabase(dco);
            }
        }
        
        try {
            initializeConstraints();
            stmt.close();
        } catch (Exception ignore) {}
    }

    /**
     * Checks if all columns of a table are present. If not, the missing columns
     * are added to the table
     * @param metaData object holding the table specifics
     * @param dcObj data crow object for which the table is created
     */
    private void initializeColumns(ResultSetMetaData metaData, DCObject dcObj) 
            throws SQLException {
        String[] columns = dcObj.getDatabaseFieldNames();
        String tableName = dcObj.getTableName();
        
        int metaCount = metaData.getColumnCount();
        if (metaCount < columns.length) {
            Messages.add(Resources.getText(DCMessages.msgTableNeedsUpgrade, tableName),
                                Messages._STANDARD);
            
            String[] metaColumns = new String[metaCount];
            for (int i = 1; i < metaCount + 1; i++) {
                metaColumns[i - 1] = metaData.getColumnName(i);
            }
            
            for (int i = 0; i < columns.length; i++) {
                boolean exists = false;
                for (int j = 0; j < metaColumns.length; j++) {
                    if (columns[i].equalsIgnoreCase(metaColumns[j])) {
                        exists = true;
                    }
                }
                if (!exists) {
                    this.runQueryDirect("alter table " + 
                                        tableName + 
                                        " add column " +
                                        dcObj.getDatabaseFieldName(i) + " " +
                                        dcObj.getField(i).getDataBaseFieldType()); 
                }
            }
        }
    }
    
    /**
     * Sets constraints on tables 
     */
    private void initializeConstraints() {
        try {
            String query1 = "ALTER TABLE audiotracks ADD CONSTRAINT AUDIOTRACK_UK " +
                            "UNIQUE (ID)"; 
            runQueryDirect(query1, false);
            
            String query2 = "ALTER TABLE audiocds ADD CONSTRAINT AUDIOCD_UK " +
                            "UNIQUE (ID)";   
            runQueryDirect(query2, false);
            
        	String query3 = "ALTER TABLE audiotracks ADD CONSTRAINT " +
                            "FK_AUDIOCDS_AUDIOTRACKS FOREIGN KEY (ALBUM) " +
                            "REFERENCES audiocds (ID) ON DELETE CASCADE";
            runQueryDirect(query3, false);
        } catch (Exception exp) {
            String msg = exp.getMessage();
            if (msg.toUpperCase().indexOf("EXISTS") < 0) {
                Messages.add(exp);
            }
        }
    }

    /**
     * Creates the tables and fields on an empty database
     * @param o the data crow object for which the table should be created
     */
    private void initializeNewDatabase(DCObject dco) {
        QueryWhat what = new QueryWhat(dco, dco.getCreateStatements());
        Query query = new Query(what, null, null, Query._CREATE, null, null);

        this.runQueryDirect(query.getQueryString());
        
        loadDefaultData(dco.getModule());
    }
    
    private void loadDefaultData(DCModule module) {
        try {
        	DCObject[] objects = module.getDefaultData();
            for (int i = 0; i < objects.length; i++) {
                objects[i].setSilence(true);
                objects[i].saveNew();
            }
        } catch (Exception exp) {
        	Messages.add(exp);
        }
    }
}
