/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.db;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;
import java.util.LinkedList;

import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.Repository;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.wf.WorkFlow;
import net.sf.dc.wf.requests.RequestCollection;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;

/**
 * Query queue with last in last out mechanism. Request for actions depending
 * on the query are past on to the WorkFlow.
 * The queue is in fact a LinkedList
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class QueryQueue extends Thread {

    private final LinkedList lQueryQueue = new LinkedList();

    private boolean bLazy = true;
    private final Connection connection;
    private final WorkFlow wf = new WorkFlow();
    private boolean success = true;
    
    /**
     * Initializes the queue, it sets the connection and the statement object
     * @param conn database connection
     */
    public QueryQueue(Connection connection) {
        this.connection = connection;
    }

    /**
     * Adds a query to the queue (at the last position)
     * @param query the query object
     */
    public void addQuery(Query query) {
        lQueryQueue.addLast(query);
    }
    
    /**
     * Starts the queue. If the queue is empty the thread is set in
     * sleep mode
     */
    public void run() {
        synchronized(this) {
            while (true) {
                try {
                    if (lQueryQueue.size() > 0 && bLazy) {
                        success = true;
                        bLazy = false;
                        Object o = lQueryQueue.removeFirst();
                        
                        Collection objects = null;
                        RequestCollection requestors = null;
                        if (o instanceof Query) {
                            Query query = (Query) o;
                            objects = executeQuery(query);
                            requestors = query.getRequestors();
                        } 
                    	updateUI(objects, requestors, success);
                        o = null;
                    } else {
                        bLazy = true;
                        sleep(100);
                    }
                } catch (Exception exp) {
                    Messages.add(exp);
                }
            }
        }
    }

    /**
     * Executes the query. All query types are executed in this method.
     * In case a result set was produced, it is returned. If no
     * result set is produced null is returned
     * @param query  the query object to be executed
     * @return (ResultSet) a vector of datacrow objects (can be null)
     */
    private Collection executeQuery(Query query) {
        switch(query.getQueryType()) {
            case Query._UPDATE:
                success = executeUpdateQuery(query);
                break;
            case Query._DELETE:
                success = executeDeleteQuery(query);
                break;                
            case Query._INSERT:
                success = executeInsertQuery(query);
                break;
            case Query._UNDEFINED:
                if (!query.getSilence()) {
                	Messages.add(query.getQueryString(), Messages._QUERY);
                }
                return executeUndefinedQuery(query);
            default:
                if (!query.getSilence()) {
                    Messages.add(query.getQueryString(), Messages._QUERY);
                }
                return executeSelectQuery(query);
        }

        if (!query.getSilence()) {
        	Messages.add(query.getQueryString(), Messages._QUERY);
        }
        return null;
    }

    /**
     * Executes an update query
     * @param query the query to be executed
     */
    private boolean executeUpdateQuery(Query query) {
        String sQuery = query.getQueryString();
        
        Statement stmt = null;
        try {
            stmt = connection.createStatement();
            stmt.executeUpdate(sQuery);
            if (!query.getSilence()) {
                new MessageBox( Resources.getText(
                    DCMessages.msgRecSaved, query.getItemID()), 
                    MessageBox._INFORMATION, false);
            } else if (query.isBatch() && query.isEndOfBatch()) {
                new MessageBox( 
                        Resources.getText(DCMessages.msgDataSaved), 
                        MessageBox._INFORMATION, false);
            }
            
            stmt.execute("commit;");
            return true;
        } catch (SQLException sqlExp) {
            Messages.add(sQuery, Messages._QUERY);
            Messages.add(sqlExp);
            new MessageBox(Resources.getText(
                DCMessages.msgQueryError, sqlExp.toString()), 
                MessageBox._ERROR, false);
            return false;
        } finally {
            closeStatement(stmt);
        }
    }
    
    /**
     * Executes an insert query
     * @param query the query to be executed
     */
    private boolean executeInsertQuery(Query query) {
        String sQuery = query.getQueryString();

        Statement stmt = null;
        try {
            stmt = connection.createStatement();
            stmt.execute(sQuery);
            if (!query.getSilence()) {
                new MessageBox( 
                    Resources.getText(DCMessages.msgRecSaved, query.getItemID()), 
                    MessageBox._INFORMATION, false);
            } else if (query.isBatch() && query.isEndOfBatch()) {
                new MessageBox( 
                        Resources.getText(DCMessages.msgDataSaved), 
                        MessageBox._INFORMATION, false);
                
            }
            
            stmt.execute("commit;");
            return true;            
        } catch (SQLException sqlExp) {
            Messages.add(sQuery, Messages._QUERY);
            Messages.add(sqlExp);
            new MessageBox(Resources.getText(
                DCMessages.msgQueryError, sqlExp.toString()), 
                MessageBox._ERROR, false);
        } finally {
            closeStatement(stmt);
        }
        return false;
    }

    /**
     * Executes am delete query
     * @param query the query to be executed
     */
    private boolean executeDeleteQuery(Query query) {
        String sQuery = query.getQueryString();
        Statement stmt = null;
        try {
            stmt = connection.createStatement();
            stmt.executeQuery(sQuery);
            if (!query.getSilence()) {
                new MessageBox(
                    Resources.getText(DCMessages.msgItemDeleted, query.getItemID()), 
                    MessageBox._INFORMATION, false);
            } else if (query.isBatch() && query.isEndOfBatch()) {
                new MessageBox( 
                        Resources.getText(DCMessages.msgDeleteSuccessfull), 
                        MessageBox._INFORMATION, false);
            }
   
            stmt.execute("commit;");
            return true;         
        } catch (SQLException sqlExp) {
            Messages.add(sQuery, Messages._QUERY);
            Messages.add(sqlExp);
            new MessageBox(
                Resources.getText(DCMessages.msgQueryError, sqlExp.toString()), 
                MessageBox._ERROR, false);
        } finally {
            closeStatement(stmt);
        }
        return false;
    }


    /**
     * Executes a select query
     * @param query the query to be executed
     */
    private Collection executeSelectQuery(Query query) {
        String sQuery = query.getQueryString();
        ResultSet result = null;

        int iMaxSearcRes = Settings.getValueAsInt(Repository.settings.stMaximumSearchRes);

        Statement stmt = null;
        try {
            stmt = connection.createStatement();
            stmt.setMaxRows(iMaxSearcRes);
            result = stmt.executeQuery(sQuery);
        } catch (SQLException sqlExp) {
            Messages.add(sQuery, Messages._QUERY);
            Messages.add(sqlExp);
            new MessageBox(
                Resources.getText(DCMessages.msgQueryError, sqlExp.toString()), 
                MessageBox._ERROR, false);
        } 
        
        Collection objects = WorkFlow.convertToDCObjects(result);
        try {
            result.close();
            closeStatement(stmt);
        } catch (SQLException exp) {
            Messages.add(exp);
        }
        return objects;
    }

    /**
     * Executes an undified query type (there is no knowledge if a resultset
     * is returned or not)
     * @param query the query to be executed
     */
    private Collection executeUndefinedQuery(Query query) {
        String sQuery = query.getQueryString();
        ResultSet result = null;

        int iMaxSearcRes = Settings.getValueAsInt(Repository.settings.stMaximumSearchRes);

        Statement stmt = null;
        try {
            stmt = connection.createStatement();
            stmt.setMaxRows(iMaxSearcRes);
            result = stmt.executeQuery(sQuery);
        } catch (SQLException sqlExp) {
            Messages.add(sQuery, Messages._QUERY);
            Messages.add(sqlExp);
            new MessageBox(
                Resources.getText(DCMessages.msgQueryError, sqlExp.toString()), 
                MessageBox._ERROR, false);
        } 
        
        Collection objects = WorkFlow.convertToDCObjects(result);
        try {
            result.close();
            closeStatement(stmt);
        } catch (SQLException exp) {
            Messages.add(exp);
        }
        return objects;        
    }

    /**
     * Passes requests and data for the UI to the Workflow layer
     * @param result result of the query
     * @param requestors requestors for actions or data
     */
    private void updateUI(Collection objects, 
                          RequestCollection requestors, 
                          boolean qryWasSuccess) {
        wf.displayResults(objects, requestors, qryWasSuccess);
    }
    
    /**
     * Closes the SQL statement
     * @param stmt the statement to be closed
     */
    private void closeStatement(Statement stmt) {
        try {
            connection.commit();
            if (stmt != null) {
                stmt.close();
                stmt = null;
            }
        } catch (SQLException exp) {
            Messages.add(exp);
        }
    }
}
