/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.http.covers;

import java.io.PrintWriter;
import java.net.URL;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sf.dc.console.InternetSearchForm;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.http.HttpConnection;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.util.Resources;

/**
 * Retrieves covers for audio CD's from the amazon webserver.
 * Uses SOAP for the communication.
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.6
 */
public class AmazonCoverSource {
    
    // the caller
    private final InternetSearchForm ui;
    private final AudioCD audioCD;
    
    private boolean keepOnRunning = true;
    
	private static final String SOAP_ACTION = "SearchArtistRequest";
	private static final String SOAP_URL = "http://soap.amazon.com/onca/soap3";
	private static final Pattern SOAP_PATTERN = Pattern.compile(".+?ProductName.+?>(.+?)</ProductName.+?Artist .+?>(.+?)</Artist.+?ImageUrlSmall.+?>(.+?)</ImageUrlSmall.+?ImageUrlMedium.+?>(.+?)</ImageUrlMedium.+?ImageUrlLarge.+?>(.+?)</ImageUrlLarge.*");
    
    private static final Pattern[] stripPatterns = {
            Pattern.compile(" ?\\(dis[ck] \\d+?\\)"),
            Pattern.compile("^(the|a|an|el|le|les|la|los|die|der|das|den) "),
            Pattern.compile(" (the|a|an|el|le|les|la|los|die|der|das|den) "),
            Pattern.compile("('|~|\\!|@|#|\\$|%|\\^|\\*|_|\\[|\\{|\\]|\\}|\\||\\\\|;|:|`|\"|<|,|>|\\.|\\?|/|&)")};    

	public AmazonCoverSource(InternetSearchForm ui, AudioCD audioCD) {
        this.ui = ui;
        this.audioCD = audioCD;
    }
    
    public void stopSearch() {
        keepOnRunning = false; 
    }
    
    /**
     * Creates a correct SOAP request which can be send to the Amazon
     * web service
     * @param artist the artist name to search for
     * @param album the album name to search for
     */
	private String getSoapRequest(String artist, String album) {
		StringBuffer sb = new StringBuffer();
		sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		sb.append("<SOAP-ENV:Envelope SOAP-ENV:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:SOAP-ENC=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:xsi=\"http://www.w3.org/1999/XMLSchema-instance\" xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsd=\"http://www.w3.org/1999/XMLSchema\">");
		sb.append("<SOAP-ENV:Body>");
		sb.append("<ns1:ArtistSearchRequest xmlns:ns1=\"http://soap.amazon.com\" SOAP-ENC:root=\"1\">");
		sb.append("<v1>" + "<sort xsi:type=\"xsd:string\">+pmrank</sort>");
		sb.append("<artist xsi:type=\"xsd:string\">").append(artist).append("</artist>");
		sb.append("<devtag xsi:type=\"xsd:string\">D1R5WXCELBX4NF</devtag>");
		sb.append("<tag xsi:type=\"xsd:string\">accessonline</tag>");
		sb.append("<mode xsi:type=\"xsd:string\">music</mode>");
		sb.append("<keywords xsi:type=\"xsd:string\">").append(album).append("</keywords>");
		sb.append("<type xsi:type=\"xsd:string\">lite</type>");
		sb.append("<page xsi:type=\"xsd:int\">1</page>" + "</v1>");
		sb.append("</ns1:ArtistSearchRequest>" + "</SOAP-ENV:Body>");
		sb.append("</SOAP-ENV:Envelope>");
		
		return sb.toString();
	}

    /**
     * Sends a SOAP request
     * @param request the SOAP request (@see #getSoapRequest(String, String))
     * @return the search result as string (XML)
     * @throws Exception
     */
	private String sendSoapRequest(String request) throws Exception {
		//Set up the request
        URL url = new URL(SOAP_URL);
		HttpConnection connection = new HttpConnection(url);
		connection.setContentLength(request.length());
		connection.setToSoapRequest(SOAP_ACTION);

		//Send our xml request
        ui.addMessage(Resources.getText(DCMessages.msgSendingSearchAmazon));
		PrintWriter out = new PrintWriter(connection.getOutputStream());
		out.println(request);
		out.close();

		//Receive the answer
        String answer = connection.getString(url);
		return answer;
	}
	
    /**
     * Searches the amazon webservice for audio CD covers.
     * Uses information (title and artist) from the Audio CD item.
     */
	public void search() {
        String album = audioCD.getValue(AudioCD._A_TITLE).toString();
        String artist = audioCD.getValue(AudioCD._B_ARTIST).toString();
        
		//Transform the artist and album strings to better search
		artist = normalize(artist);
		album = normalize(album);

        String answer;
        try {
        	answer = sendSoapRequest(getSoapRequest(artist, album));
        	answer = answer.replaceAll("\n", "");
        	answer = answer.replaceAll("&amp;", "&");
        } catch (Exception exp) {
        	ui.addMessage(Resources.getText(DCMessages.msgCouldNotConnectAmazon, exp.getMessage()));
            Messages.add(exp);
            return;
        }
        
		int nextSubIndex = 0;
        Vector vURLs = new Vector(0);
		while (answer.length() > nextSubIndex && keepOnRunning) {
			Matcher matcher = SOAP_PATTERN.matcher(answer.substring(nextSubIndex));
			if (matcher.matches()) {
				//Retrieve the best url (non-empty image)
				String imageURL = "";
				for (int i = 5; i >= 3; i--) {
					imageURL = matcher.group(i);
					if (!imageURL.equals("")) {
                        vURLs.add(imageURL);
                        break;
                    }
				}
				nextSubIndex += matcher.end(matcher.groupCount());
			} else {
				break;
            }
		}
        retrieveImages(vURLs);
	}
    
    /**
     * Retrieves the images from the provided URLs.
     * For each recovered cover a new item is added to the internet
     * search form
     * 
     * @param vURLs Vector of URLs
     */
    private void retrieveImages(Vector vURLs) {
        ui.addMessage(Resources.getText(DCMessages.msgDownloadingXImages, String.valueOf(vURLs.size())));
        
        int results = vURLs.size();
        ui.initProgressBar(results);
        if (results == 0) {
        	ui.initProgressBar(1);
            ui.updateProgressBar(1);
            return;
        }
        
        int counter = 0;
        for (Enumeration enum = vURLs.elements(); enum.hasMoreElements(); counter++) {
            try {
                URL url = new URL(enum.nextElement().toString());
                HttpConnection connection = new HttpConnection(url);
                // check if this image is empty or not
                if (connection.getContentLength() != 807) {
                    // In case more then one image is found, add the item 
                    // again with the new image
                    if (counter > 0) {
                        AudioCD cd = new AudioCD();
                        Object[] values = audioCD.getValues();
                        
                        cd.setValues(values);
                        Collection children = audioCD.getChildren();
                        for (Iterator iter = children.iterator(); iter.hasNext(); ) {
                        	cd.addChild((DCObject) iter.next());
                        }
                        
                        cd.setValue(AudioCD._E_PICTUREFRONT, connection.getImageAsBase64());                        
                        if (counter == 1) {
                        	ui.addMessage(Resources.getText(DCMessages.msgMoreThenOneImageFound));
                        }
                    	ui.addObject(cd);
                    } else {
                        audioCD.setValue(AudioCD._E_PICTUREFRONT, connection.getImageAsBase64());
                        ui.addMessage(Resources.getText(DCMessages.msgImageRetrieved));
                    }
                } else {
                    ui.addMessage(Resources.getText(DCMessages.msgSkippingEmptyImage));
                }
            } catch (Exception exp) {
                ui.addMessage(Resources.getText(DCMessages.msgErrorWithImage, exp.getMessage()));
                Messages.add(exp);
            }
            ui.updateProgressBar(counter + 1);
        }
    }
    
    /**
     * Strips characters not belonging in a SOAP request
     * @param s string to normalize
     * @return normalized string
     */
    public static String normalize(String s) {
        s = s.toLowerCase();

        for (int i = 0; i < stripPatterns.length; i++) {
            Matcher ma = stripPatterns[i].matcher(s);
            s = ma.replaceAll("");
        }

        return s;
    }
}
