/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Enumeration;
import java.util.Vector;

import net.sf.dc.console.musicfile.organizer.MusicFileMoverPanel;
import net.sf.ext.util.DirectoryReader;
import entagged.audioformats.AudioFile;
import entagged.audioformats.AudioFileIO;
import entagged.audioformats.Tag;

/**
 * @author Robert Jan van der Waals
 * @version 1.5
 * @since 1.4
 */
public class MusicFilesOrganizerTask extends Thread {

    private String sSourceDir;
    private String sTargetDir;

    private boolean  bSameTarget;
    private String[] sortBy;
    
    private boolean keepOnRunning = true;
    private final MusicFileMoverPanel ui;
    
    String[] extensions = new String[] {"mp3", "ogg", "mp4", "flac", "flc", "ape", "asf", "wav", "mpc"};

    public void stopRunning() {
    	this.keepOnRunning = false;
    }
    
    public MusicFilesOrganizerTask(MusicFileMoverPanel ui, 
                                   String[] sortBy, 
                                   String sSourceDir, 
                                   String sTargetDir) {
        
        this.ui = ui;
        if (sortBy == null || sortBy.length < 1) {
            return;
        }

        this.sortBy = sortBy;
        this.sSourceDir = sSourceDir;
        if (sTargetDir == null || sTargetDir.trim().equals("")) {
            bSameTarget = true;
        } else {
            this.sTargetDir = sTargetDir;
            bSameTarget = false;
        }
    }

    /**
     * Starts the process
     */
    public void run() {
        if (sortBy == null) {
            return;
        }

        for (int i = 0; i < sortBy.length; i++) {
            ui.updateLog("Starting to organize");
            boolean byArtist = sortBy[i].toLowerCase().equals("artist");
            boolean byAlbum = sortBy[i].toLowerCase().equals("album");
            boolean byGenre = sortBy[i].toLowerCase().equals("genre");
            start(byArtist, byAlbum, byGenre);
        }
        ui.updateLog("Organizing has finished");
    }

    /**
     * Searches for Music files in the source location and moves them to the target
     * location
     *
     * @param byArtist sort by artist
     * @param byAlbum sort by album
     * @param byTitle sort by title
     */
    private void start(boolean byArtist, boolean byAlbum, boolean byGenre) {
        try {
            
            Vector files = DirectoryReader.read(sSourceDir, false, true, false, extensions, false);
            if (files.size() == 1) {
                ui.updateLog("No files found to organize!");
            } else {
                moveFiles(files, byArtist, byAlbum, byGenre);
            }
        } catch (Exception exp) {
            ui.updateLog("Error while trying to retrieve the files: " + exp.getMessage());
        }
    }

    /**
     * Moves the Audio files to the target directory + sort criteria value
     *
     * @param vFiles    mp3 file locations
     * @param byArtist  sort by artist
     * @param byAlbum   sort by album
     * @param byTitle   sort by title
     */
    private void moveFiles(Vector vFiles, boolean byArtist, boolean byAlbum, boolean byGenre) {
            ui.initProgressBar(vFiles.size());
            
            int progress = 1;
            keepOnRunning = true;
            for (Enumeration enum = vFiles.elements(); enum.hasMoreElements(); progress++) {
                File file = new File(enum.nextElement().toString());
                String filename = file.toString();
                try {
                    if (!keepOnRunning) {
                        return;
                    }
                    sSourceDir = filename.substring(0, ((filename.length()-1) - file.getName().length()));
                    boolean bMoved       = false;
                    String  sDirectory   = null;

                    if (file.isFile() && !file.isDirectory()) {
                        AudioFile audioFile = AudioFileIO.read(file);
                        Tag tag = audioFile.getTag();
                        String sAlbum = tag.getAlbum().toString();
                        String sArtist = tag.getArtist().toString();
                        String sGenre = tag.getGenre().toString();

                        String newFilename = "";
                        if (byArtist && (sArtist != null && !sArtist.trim().equals(""))) {
                            sDirectory  = createDir(sArtist);
                            String sNewFile    = sDirectory + File.separator + file.getName();
                            bMoved      = moveFile(filename, sNewFile);
                        } else if (byAlbum && (sAlbum != null && !sAlbum.trim().equals(""))) {
                            sDirectory = createDir(sAlbum);
                            newFilename = sDirectory + File.separator + file.getName();
                            bMoved = moveFile(filename, newFilename);
                        } else if (byGenre && (sGenre != null && !sGenre.trim().equals(""))) {
                            sDirectory = createDir(sGenre);
                            newFilename = sDirectory + File.separator + file.getName();
                            bMoved = moveFile(filename, newFilename);
                        }
                    }

                    if (bMoved) {
                        ui.updateLog("Moved file <" + filename + "> To <" + sDirectory + ">");
                        file.delete();
                    } else {
                        ui.updateLog("Not enough information in the tag to move file <" + filename + ">");
                    }
                } catch (Exception exp) {
                    ui.updateLog("Error moving file <" + filename + ">, reason:" + exp.getMessage()); 
                } finally {
                    ui.updateProgressBar(progress);
                }
            }
            keepOnRunning = false;
            ui.finish();
    }

    /**
     * Moves one mp3 file to the target directory
     * @param sourceFile  the old filename (absolute path)
     * @param targetFile  new filename (absolute path)
     */
    private boolean moveFile(String sourceFile, String targetFile) {
        try {
            try {
            	sleep(1000l);
            } catch (Exception ignore) {}
            
            final FileInputStream  fis = new FileInputStream(sourceFile);
            final FileOutputStream fos = new FileOutputStream(targetFile);
            byte[]  b = new byte[fis.available()];
            int     k = 0;
            do {
                k = fis.read(b);
                if (k != -1) {
                    fos.write(b);
                }
            } while (k > 0);

            fos.close();
            fis.close();
            
            sleep(1000l);
        } catch (Exception exp) {
            return false;
        }
        return true;
    }

    /**
     * Creates a directory
     * @param sDirectory new directory name
     */
    private String createDir(String sDirectory) {
        File    fDirectory  = null;
        try {
            if (bSameTarget) {
                fDirectory = new File(sSourceDir);
                fDirectory.mkdir();

                fDirectory  = new File(sSourceDir + File.separator + sDirectory.trim());
                fDirectory.mkdir();
            } else {
                fDirectory = new File(sTargetDir);
                fDirectory.mkdir();

                fDirectory  = new File(sTargetDir + File.separator + sDirectory.trim());
                fDirectory.mkdir();
            }
        } catch (Exception exp) {
            exp.printStackTrace();
        }
        return fDirectory.getAbsolutePath();
    }
}
