package net.sf.ext.mediamanager;

import java.io.DataInputStream;
import java.io.RandomAccessFile;

import entagged.audioformats.asf.data.AsfHeader;
import entagged.audioformats.asf.data.StreamBitratePropertiesChunk;
import entagged.audioformats.asf.data.VideoStreamChunk;
import entagged.audioformats.asf.io.AsfHeaderReader;

/**
 * This class provides a FileProperties extension for the use of ASF media
 * files.<br>
 * It would have been nice if it also use a DataInputStream in order to read the
 * contents. However, since it uses the "Entagged"-Audio-Library a
 * RandomAccessFile is mandatory.
 * 
 * @author Christian Laireiter
 */
public class FilePropertiesASF extends FileProperties {

	/**
	 * Reflects, whether the {@link #process(RandomAccessFile)} has found a
	 * video stream or not.
	 */
	private boolean videoStreamPresent = true;

	/**
	 * Returns <code>true</code> if the last call of
	 * {@link #process(RandomAccessFile)} has found a video stream.<br>
	 * Only the this objects fields have been refilled.
	 * 
	 * @return <code>true</code> if video stream was found.
	 */
	public boolean containsVideoStream() {
		return this.videoStreamPresent;
	}

	/**
	 * (overridden)
	 * 
	 * @see net.sf.ext.mediamanager.FileProperties#process(java.io.DataInputStream)
	 */
	public void process(DataInputStream dataStream) throws Exception {
		// Will be Ignroed
	}

	/**
	 * This method will read the file and interprets it as an asf-media-file.<br>
	 * 
	 * @param file
	 *            The asf media file, which should be parsed.
	 * @throws Exception
	 *             Thrown on IOErrors, or failure in interpreting the file as
	 *             asf.
	 */
	public void process(RandomAccessFile file) throws Exception {
		// Let the audio library read the asf file.
		AsfHeader header = AsfHeaderReader.readHeader(file);

		VideoStreamChunk video = null;
		int streamCount = header.getStreamChunkCount();
		/*
		 * The first found video-stream will be taken for making up the
		 * properties object.
		 */
		for (int i = 0; i < streamCount; i++) {
			if (header.getStreamChunk(i) instanceof VideoStreamChunk) {
				video = (VideoStreamChunk) header.getStreamChunk(i);
			}
		}
		this.videoStreamPresent = video != null;
		// Only fill up the rest, if a video stream has been found
		if (this.videoStreamPresent) {
			setVideoWidth((int) video.getPictureWidth());
			setVideoHeight((int) video.getPictureHeight());
			setVideoResolution(getVideoWidth() + "x" + getVideoHeight());
			setVideoCodec(video.getCodecIdAsString());
			/*
			 * The fps of the video can't be determined yet. The
			 * entagged-library would need to interpret video data itself, which
			 * depends on the used codec.
			 */
			// setVideoFps(0);
			setDuration(header.getFileHeader().getDurationInSeconds());
			/*
			 * The average bitrate of the video is as hard to gather as the FPS.
			 * However in this case there is a recommended chunk in asf files
			 * which contains this information.
			 */
			StreamBitratePropertiesChunk propertiesChunk = header
					.getStreamBitratePropertiesChunk();
			if (propertiesChunk != null) {
				setVideoBitRate((int) propertiesChunk.getAvgBitrate(video
						.getStreamNumber()));
			}
			/*
			 * Now the audio part of the video. (optional)
			 */
			if (header.getAudioStreamChunk() != null) {
				setAudioCodec(new String(header.getAudioStreamChunk()
						.getCodecDescription()));
				setAudioRate((int) header.getAudioStreamChunk()
						.getSamplingRate());
				setAudioBitRate(header.getAudioStreamChunk().getKbps() * 1000);
				setAudioChannels((int) header.getAudioStreamChunk()
						.getChannelCount());
			}
		}
	}
}
