package net.sf.ext.mediamanager;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.RandomAccessFile;
import java.util.Arrays;

import entagged.audioformats.asf.data.GUID;

public class FilePropertiesMovie {

    /* The filesize */
    private int _fileSize = -1;
    /* The resolution (width x heigth) */
    private String _videoResolution = "";
    /* The video codec (vids handler) */
    private String _videoCodec = "";
    /* The video rate (fps) */
    private int _videoFps = -1;
    /* The video bit rate (kbps) */
    private int _videoBitRate = -1;
    /* The video duration (seconds) */
    private int _duration = -1;
    /* The audio codec (auds handler) */
    private String _audioCodec = "";
    /* The audio rate (Hz) */
    private int _audioSampleRate = -1;
    /* The audio bit rate (kbps) */
    private int _audioBitRate = -1;
    /* The audio channels */
    private int _audioChannels = -1;
    private int _videoWidth = -1;
    private int _videoHeight = -1;
    
    private String _name = "";

    /**
     * The Constructor. Gets all info available (reads contents from file).
     *
     * @param filePath A path for the file.
     **/
    public FilePropertiesMovie(String filePath) throws Exception {
        final int[][] MAGIC_BYTES = {{0x00, 0x00, 0x01, 0xb3 }, // MPEG (video)
                                     {0x00, 0x00, 0x01, 0xba }, // MPEG (video)
                                     {0x52, 0x49, 0x46, 0x46 }, // RIFF (WAV / audio, AVI / video)
                                     {0x30, 0x26, 0xb2, 0x75}}; // ASF (audio / video)
        
        final FileProperties[] FORMATS = {  new FilePropertiesMPEG(),
                                            new FilePropertiesMPEG(),
                                            new FilePropertiesRIFF(),
                                            new FilePropertiesASF()};

        _fileSize = Math.round((new File(filePath).length()) / 1024F / 1024F);
        DataInputStream dataStream = new DataInputStream(new BufferedInputStream(
                                                         new FileInputStream(filePath)));

        dataStream.mark(4);
        int[] header = new int[4];
        for (int i = 0; i < header.length; i++) {
            header[i] = dataStream.readUnsignedByte();
        }

        File temp = new File(filePath);
        String name = temp.getName().substring(0, temp.getName().lastIndexOf("."));
        name = name.replaceAll(".", " ");
        name = name.replaceAll("_", " ");
        name = name.replaceAll("-", " ");
        
        setVideoName(name);

        int format;
        for (format = 0; format < FORMATS.length; format++) {
            if (Arrays.equals(header, MAGIC_BYTES[format])) {
                /*
                 * Now comes a special addition. Since know 4 bytes where 
                 * sufficient. Now the first 4 bytes of the ASF-Header GUID
                 * were identified. If (according to prior specification at 
                 * magic_bytes) the current index is the last registered format,
                 * it must be verified, that all other GUID bytes match. 
                 */
                if (format == FORMATS.length-1) {
                    int[] guid = new int[GUID.GUID_LENGTH];
                    // Copy already read bytes into the beginning of guid.
                    for (int i = 0; i < header.length; i++) {
                        guid[i] = header[i] & 0xFF;
                    }
                    // Read the remaining bytes to fill up a ASF-GUID.s
                    for (int i = header.length; i < GUID.GUID_LENGTH; i++) {
                        guid[i] = dataStream.readUnsignedByte();
                    }
                    // Compare them.
                    if (GUID.GUID_HEADER.equals(new GUID(guid))) {
                        break;
                    }
                } else {
                    break;
                }
            }
        }

        if (format < FORMATS.length) {
            FileProperties fileProperties = FORMATS[format];
            fileProperties.process(dataStream);
            
            if (fileProperties instanceof FilePropertiesASF) {
            	/*
            	 * For FilePropertiesASF the process(DataInputStream) is empty.
            	 * It needs a RandomAccessFile to perform its task.
            	 */
            	((FilePropertiesASF) fileProperties).process(
            			new RandomAccessFile(filePath,"r"));
            }             
            
            _videoResolution = fileProperties.getVideoResolution();
            _videoWidth = fileProperties.getVideoWidth();
            _videoHeight = fileProperties.getVideoHeight();
            _videoCodec = fileProperties.getVideoCodec();
            _videoFps = fileProperties.getVideoFps();
            _videoBitRate = fileProperties.getVideoBitRate();
            _duration = fileProperties.getDuration();
            _audioCodec = fileProperties.getAudioCodec();
            _audioSampleRate = fileProperties.getAudioSampleRate();
            _audioBitRate = fileProperties.getAudioBitRate();
            _audioChannels = fileProperties.getAudioChannels();
        }
        dataStream.close();
    }

    /**
     * Returns the filesize.
     */
    public int getFileSize() {
        return _fileSize;
    }

    public String getVideoName() {
        return _name;
    }

    /**
     * Returns the resolution.
     */
    public String getVideoResolution() {
        return _videoResolution;
    }

    public int getVideoHeight() {
        return _videoHeight;
    }
    
    public int getVideoWidth() {
        return _videoWidth;
    }

    /**
     * Returns the video codec (vids handler).
     */
    public String getVideoCodec() {
        return _videoCodec;
    }

    /**
     * Returns the video rate.
     */
    public int getVideoFps() {
        return _videoFps;
    }

    /**
     * Returns the video bit rate.
     */
    public int getVideoBitRate() {
        return _videoBitRate;
    }

    /**
     * Returns the duration.
     */
    public int getDuration() {
        return _duration;
    }

    /**
     * Returns the audio codec (auds handler).
     */
    public String getAudioCodec() {
        return _audioCodec;
    }

    /**
     * Returns the audio rate.
     */
    public int getAudioSampleRate() {
        return _audioSampleRate;
    }

    /**
     * Returns the audio bit rate.
     */
    public int getAudioBitRate() {
        return _audioBitRate;
    }

    /**
     * Returns the audio channels.
     */
    public int getAudioChannels() {
        return _audioChannels;
    }

    /**
     * Sets the length.
     */
    public void setFileSize(int fileSize) {
        _fileSize = fileSize;
    }

    /**
     * Sets the resolution.
     */
    public void setVideoResolution(String videoResolution) {
        _videoResolution = videoResolution;
    }

    /**
     * Sets the name 
     */
    public void setVideoName(String name) {
        _name = name;
    }

    /**
     * Sets the video codec (vids handler).
     */
    public void setVideoCodec(String videoCodec) {
        _videoCodec = videoCodec;
    }

    /**
     * Sets the video rate.
     */
    public void setVideoRate(int fps) {
        _videoFps = fps;
    }

    /**
     * Sets the duration.
     */
    public void setDuration(int duration) {
        _duration = duration;
    }

    /**
     * Sets the audio codec (auds handler).
     */
    public void setAudioCodec(String audioCodec) {
        _audioCodec = audioCodec;
    }

    /**
     * Sets the audio rate.
     **/
    public void setAudioRate(int audioSampleRate) {
        _audioSampleRate = audioSampleRate;
    }

    /**
     * Sets the audio channels.
     */
    public void setAudioChannels(int audioChannels) {
        _audioChannels = audioChannels;
    }
}
